package com.example.demo.comments.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.repository.CommentRepository;
import com.example.demo.core.error.NotFoundException;

@Service
public class CommentService {
    private final CommentRepository repository;

    public CommentService(CommentRepository repository) {
        this.repository = repository;
    }

    public List<CommentEntity> getAll() {
        return repository.getAll();
    }

    public CommentEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public CommentEntity create(CommentEntity entity) {
        return repository.create(entity);
    }

    public CommentEntity update(Long id, CommentEntity entity) {
        final CommentEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        return repository.update(existsEntity);
    }

    public CommentEntity delete(Long id) {
        final CommentEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
