package com.example.demo.messages.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.convert;
import com.example.demo.core.configuration.Constants;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/message")
public class MessageController {
    private final UserService userService;
    private final MessageService messageService;
    private final ModelMapper modelMapper;

    public MessageController(MessageService messageService, UserService userService, ModelMapper modelMapper) {
        this.messageService = messageService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private MessageDto toDto(MessageEntity entity) {
        var model = modelMapper.map(entity, MessageDto.class);
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private MessageEntity toEntity(MessageDto dto) {
        final MessageEntity entity = modelMapper.map(dto, MessageEntity.class);
        entity.setFriend(userService.get(dto.getFriendId()));
        entity.setPerson(userService.get(dto.getPersonId()));
        return entity;

    }

    @GetMapping
    public List<MessageDto> getAll() {
        return messageService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public MessageDto get(@PathVariable(name = "id") Long id) {
        return toDto(messageService.get(id));
    }

    @PostMapping
    public MessageDto create(@RequestBody @Valid MessageDto dto) {
        return toDto(messageService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public MessageDto update(@PathVariable(name = "id") Long id, @RequestBody MessageDto dto) {
        return toDto(messageService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public MessageDto delete(@PathVariable(name = "id") Long id) {
        return toDto(messageService.delete(id));
    }
}
