package com.example.demo.messages.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.repository.MessageRepository;

@Service
public class MessageService {
    private final MessageRepository repository;

    public MessageService(MessageRepository repository) {
        this.repository = repository;
    }

    public List<MessageEntity> getAll() {
        return repository.getAll();
    }

    public MessageEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public MessageEntity create(MessageEntity entity) {
        return repository.create(entity);
    }

    public MessageEntity update(Long id, MessageEntity entity) {
        final MessageEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        return repository.update(existsEntity);
    }

    public MessageEntity delete(Long id) {
        final MessageEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
