package com.example.demo.posts.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.convert;
import com.example.demo.core.configuration.Constants;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.service.PostService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/post")
public class PostController {
    private final UserService userService;
    private final PostService postService;
    private final ModelMapper modelMapper;

    public PostController(PostService postService, UserService userService, ModelMapper modelMapper) {
        this.postService = postService;
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    private PostDto toDto(PostEntity entity) {
        var model = modelMapper.map(entity, PostDto.class);
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private PostEntity toEntity(PostDto dto) {
        final PostEntity entity = modelMapper.map(dto, PostEntity.class);
        entity.setPerson(userService.get(dto.getPersonId()));
        return entity;

    }

    @GetMapping
    public List<PostDto> getAll() {
        return postService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public PostDto get(@PathVariable(name = "id") Long id) {
        return toDto(postService.get(id));
    }

    @PostMapping
    public PostDto create(@RequestBody @Valid PostDto dto) {
        return toDto(postService.create(toEntity(dto)));
    }

    @PostMapping("/like/{id}")
    public PostDto addLike(@PathVariable(name = "id") Long id) {
        return toDto(postService.addLike(id));
    }

    @DeleteMapping("/like/{id}")
    public PostDto removeLike(@PathVariable(name = "id") Long id) {
        return toDto(postService.removeLiked(id));
    }

    @PostMapping("/view/{id}")
    public PostDto addView(@PathVariable(name = "id") Long id) {
        return toDto(postService.addView(id));
    }

    @PutMapping("/{id}")
    public PostDto update(@PathVariable(name = "id") Long id, @RequestBody PostDto dto) {
        return toDto(postService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public PostDto delete(@PathVariable(name = "id") Long id) {
        return toDto(postService.delete(id));
    }
}
