package com.example.demo.posts.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.repository.PostRepository;

@Service
public class PostService {
    private final PostRepository repository;

    public PostService(PostRepository repository) {
        this.repository = repository;
    }

    public List<PostEntity> getAll() {
        return repository.getAll();
    }

    public PostEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public PostEntity create(PostEntity entity) {
        return repository.create(entity);
    }

    public PostEntity addLike(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setLiked((existsEntity.getLiked() + 1));
        return repository.update(existsEntity);
    }

    public PostEntity removeLiked(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setLiked((existsEntity.getLiked() - 1));
        return repository.update(existsEntity);
    }

    public PostEntity addView(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setViewed((existsEntity.getViewed() + 1));
        return repository.update(existsEntity);
    }

    public PostEntity update(Long id, PostEntity entity) {
        final PostEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        return repository.update(existsEntity);
    }

    public PostEntity delete(Long id) {
        final PostEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
