package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;

@Service
public class UserService {
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    public List<UserEntity> getAll() {
        return repository.getAll();
    }

    public UserEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public UserEntity create(UserEntity entity) {
        return repository.create(entity);
    }

    public UserEntity addFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.getFriends().add(entity);
        return repository.update(existsEntity);
    }

    public UserEntity removeFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.getFriends().remove(entity);
        return repository.update(existsEntity);
    }

    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.setNickname(entity.getNickname());
        existsEntity.setEmail(entity.getEmail());
        existsEntity.setPassword(entity.getPassword());
        return repository.update(existsEntity);
    }

    public UserEntity delete(Long id) {
        final UserEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
