package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.posts.service.PostService;;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class CommentServiceTests {
        @Autowired
        private UserService userService;
        @Autowired
        private PostService postService;
        @Autowired
        private CommentService commentService;

        @Test
        void getTest() {
                Assertions.assertThrows(NotFoundException.class, () -> commentService.get(0L));
        }

        @Test
        @Order(1)
        void createTest() {
                var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
                var user2 = userService.create(new UserEntity(null, "eafqr", "aasd@asd", "1234r", null));
                var user3 = userService.create(new UserEntity(null, "asf", "aaasfsfs@asd", "332a", null));
                var post1 = postService.create(new PostEntity(null, user1, new Date(), "test",
                                Long.valueOf(0), Long.valueOf(0)));
                var post2 = postService
                                .create(new PostEntity(null, user3, new Date(), "asfgczv", Long.valueOf(20),
                                                Long.valueOf(15)));
                var post3 = postService.create(new PostEntity(null, user2, new Date(), "asvbsfd", Long.valueOf(30),
                                Long.valueOf(3)));
                commentService.create(new CommentEntity(null, user1, post2, new Date(), "my comment"));
                commentService.create(new CommentEntity(null, user3, post1, new Date(), "my comment2"));
                final var last = commentService
                                .create(new CommentEntity(null, user2, post3, new Date(), "my comment3"));
                Assertions.assertEquals(3, commentService.getAll().size());
                Assertions.assertEquals(last, commentService.get(3L));
        }

        @Test
        @Order(2)
        void updateTest() {
                final String test = "TEST";
                final CommentEntity entity = commentService.get(3L);
                final String oldText = entity.getText();
                var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
                var post1 = postService.create(new PostEntity(null, user1,
                                new Date(), "test",
                                Long.valueOf(0), Long.valueOf(0)));

                final CommentEntity newEntity = commentService.update(3L,
                                new CommentEntity(1L, user1, post1, new Date(), test));
                Assertions.assertEquals(3, commentService.getAll().size());
                Assertions.assertEquals(newEntity, commentService.get(3L));
                Assertions.assertEquals(test, newEntity.getText());
                Assertions.assertNotEquals(oldText, newEntity.getText());
        }

        @Test
        @Order(3)
        void deleteTest() {
                commentService.delete(3L);
                Assertions.assertEquals(2, commentService.getAll().size());
                final CommentEntity last = commentService.get(2L);
                Assertions.assertEquals(2L, last.getId());
                final var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
                final var post1 = postService.create(new PostEntity(null, user1,
                                new Date(), "test",
                                Long.valueOf(0), Long.valueOf(0)));
                final CommentEntity newEntity = commentService.create(
                                new CommentEntity(null, user1, post1, new Date(), "jgdssd"));
                Assertions.assertEquals(3, commentService.getAll().size());
                Assertions.assertEquals(4L, newEntity.getId());
        }
}
