package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class MessageServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private MessageService messageService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> messageService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity(null, "eafqr", "aasd@asd", "1234r", null));
        var user3 = userService.create(new UserEntity(null, "asf", "aaasfsfs@asd", "332a", null));
        messageService.create(new MessageEntity(null, user1, user2, new Date(), "test"));
        messageService.create(new MessageEntity(null, user3, user2, new Date(), "asfgczv"));
        final MessageEntity last = messageService
                .create(new MessageEntity(null, user2, user1, new Date(), "asvbsfd"));
        Assertions.assertEquals(3, messageService.getAll().size());
        Assertions.assertEquals(last, messageService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String test = "TEST";
        final MessageEntity entity = messageService.get(3L);
        final String oldText = entity.getText();
        final MessageEntity newEntity = messageService.update(3L,
                new MessageEntity(1L,
                        new UserEntity(null, "safsaf", "aasd@asd", "qazq", null),
                        new UserEntity(null, "safnkkjaslf", "safsaf@asd", "2qhf", null),
                        new Date(), test));
        Assertions.assertEquals(3, messageService.getAll().size());
        Assertions.assertEquals(newEntity, messageService.get(3L));
        Assertions.assertEquals(test, newEntity.getText());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    @Order(3)
    void deleteTest() {
        messageService.delete(3L);
        Assertions.assertEquals(2, messageService.getAll().size());
        final MessageEntity last = messageService.get(2L);
        Assertions.assertEquals(2L, last.getId());
        var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity(null, "sasfardq", "asfg@asd", "asfsaf", null));
        final MessageEntity newEntity = messageService.create(
                new MessageEntity(null, user1, user2, new Date(), "test"));
        Assertions.assertEquals(3, messageService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
