package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.users.service.UserService;
import com.example.demo.posts.service.PostService;;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class PostServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private PostService postService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> postService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity(null, "eafqr", "aasd@asd", "1234r", null));
        var user3 = userService.create(new UserEntity(null, "asf", "aaasfsfs@asd", "332a", null));
        postService.create(new PostEntity(null, user1,
                new Date(), "test",
                Long.valueOf(0), Long.valueOf(0)));
        postService.create(new PostEntity(null, user3, new Date(), "asfgczv", Long.valueOf(20), Long.valueOf(15)));
        final PostEntity last = postService
                .create(new PostEntity(null, user2,
                        new Date(), "asvbsfd", Long.valueOf(30),
                        Long.valueOf(3)));
        postService.addLike(1L);
        postService.addView(1L);
        Assertions.assertEquals(3, postService.getAll().size());
        Assertions.assertEquals(last, postService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String test = "TEST";
        final PostEntity entity = postService.get(3L);
        final String oldText = entity.getText();
        final PostEntity newEntity = postService.update(3L,
                new PostEntity(1L, new UserEntity(1L, "safsaf", "aasd@asd", "qazq", null),
                        new Date(), test, Long.valueOf(30), Long.valueOf(3)));
        Assertions.assertEquals(3, postService.getAll().size());
        Assertions.assertEquals(newEntity, postService.get(3L));
        Assertions.assertEquals(test, newEntity.getText());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    @Order(3)
    void deleteTest() {
        postService.delete(3L);
        Assertions.assertEquals(2, postService.getAll().size());
        final PostEntity last = postService.get(2L);
        postService.removeLiked(1L);
        Assertions.assertEquals(2L, last.getId());
        var user1 = userService.create(new UserEntity(null, "sardq", "as2@asd", "asdsd", null));
        final PostEntity newEntity = postService.create(
                new PostEntity(null, user1, new Date(), "test", Long.valueOf(0), Long.valueOf(0)));
        Assertions.assertEquals(3, postService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
