package com.example.demo;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class UserServiceTests {
    @Autowired
    private UserService userService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        List<UserEntity> friends = new ArrayList<UserEntity>();
        friends.add(new UserEntity(2L, "eafqr", "aasd@asd", "1234r", new ArrayList<UserEntity>()));
        userService.create(new UserEntity(1L, "sardq", "as2@asd", "asdsd", friends));
        userService.create(new UserEntity(2L, "eafqr", "aasd@asd", "1234r", new ArrayList<UserEntity>()));
        final UserEntity last = userService.create(new UserEntity(3L, "fgwrewr", "sf123@asd", "zxvb",
                new ArrayList<UserEntity>()));
        userService.addFriend(1L, last);
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(last, userService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String test = "TEST";
        final UserEntity entity = userService.get(3L);
        final String oldName = entity.getNickname();
        final UserEntity newEntity = userService.update(3L, new UserEntity(1L, test, "aasd@asd", "qazq", null));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(newEntity, userService.get(3L));
        Assertions.assertEquals(test, newEntity.getNickname());
        Assertions.assertNotEquals(oldName, newEntity.getNickname());
    }

    @Test
    @Order(3)
    void deleteTest() {
        userService.delete(3L);
        Assertions.assertEquals(2, userService.getAll().size());
        final UserEntity last = userService.get(2L);
        userService.removeFriend(2L, last);
        Assertions.assertEquals(2L, last.getId());
        final UserEntity newEntity = userService.create(new UserEntity(null, "asfzxv", "aasd@asd", "qwtg", null));
        Assertions.assertEquals(3, userService.getAll().size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
