package com.example.demo.comments.api;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.convert;
import com.example.demo.core.api.PageDto;
import com.example.demo.core.api.PageDtoMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.posts.service.PostService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/comment")
public class CommentController {
    private final UserService userService;
    private final PostService postService;
    private final CommentService commentService;
    private final ModelMapper modelMapper;

    public CommentController(CommentService commentService, UserService userService, PostService postService,
            ModelMapper modelMapper) {
        this.userService = userService;
        this.commentService = commentService;
        this.postService = postService;
        this.modelMapper = modelMapper;
    }

    private CommentDto toDto(CommentEntity entity) {
        var model = modelMapper.map(entity, CommentDto.class);
        convert time = new convert();
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private CommentEntity toEntity(CommentDto dto) {
        final CommentEntity entity = modelMapper.map(dto, CommentEntity.class);
        entity.setperson(userService.get(dto.getUserId()));
        entity.setPost(postService.get(dto.getPostId()));
        return entity;
    }

    @GetMapping
    public PageDto<CommentDto> getAll(
            @RequestParam(name = "page", defaultValue = "0") int page,
            @RequestParam(name = "size", defaultValue = Constants.DEFAULT_PAGE_SIZE) int size) {
        return PageDtoMapper.toDto(commentService.getAll(page, size), this::toDto);
    }

    @GetMapping("/{id}")
    public CommentDto get(@PathVariable(name = "id") Long id) {
        return toDto(commentService.get(id));
    }

    @PostMapping
    public CommentDto create(@RequestBody @Valid CommentDto dto) {
        return toDto(commentService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public CommentDto update(@PathVariable(name = "id") Long id, @RequestBody CommentDto dto) {
        return toDto(commentService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public CommentDto delete(@PathVariable(name = "id") Long id) {
        return toDto(commentService.delete(id));
    }
}
