package com.example.demo.comments.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.repository.CommentRepository;
import com.example.demo.core.error.NotFoundException;

@Service
public class CommentService {
    private final CommentRepository repository;

    public CommentService(CommentRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<CommentEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();

    }

    @Transactional(readOnly = true)
    public Page<CommentEntity> getAll(int page, int size) {
        return repository.findAll(PageRequest.of(page, size));
    }

    @Transactional(readOnly = true)
    public CommentEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(CommentEntity.class, id));
    }

    @Transactional
    public CommentEntity create(CommentEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public CommentEntity update(Long id, CommentEntity entity) {
        final CommentEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public CommentEntity delete(Long id) {
        final CommentEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
