package com.example.demo.messages.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.repository.MessageRepository;

@Service
public class MessageService {
    private final MessageRepository repository;

    public MessageService(MessageRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<MessageEntity> getAll(long personId) {
        if (personId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else {
            return repository.findByPersonId(personId);
        }

    }

    @Transactional(readOnly = true)
    public Page<MessageEntity> getAll(long personId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        if (personId <= 0L) {
            return repository.findAll(pageRequest);
        }
        return repository.findByPersonId(personId, pageRequest);
    }

    @Transactional
    public MessageEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(MessageEntity.class, id));
    }

    @Transactional
    public MessageEntity create(MessageEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public MessageEntity update(Long id, MessageEntity entity) {
        final MessageEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public MessageEntity delete(Long id) {
        final MessageEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
