package com.example.demo.posts.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.repository.PostRepository;

@Service
public class PostService {
    private final PostRepository repository;

    public PostService(PostRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<PostEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public Page<PostEntity> getAll(int page, int size) {
        return repository.findAll(PageRequest.of(page, size));
    }

    @Transactional(readOnly = true)
    public PostEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(PostEntity.class, id));
    }

    @Transactional
    public PostEntity create(PostEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public PostEntity addLike(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setLiked((existsEntity.getLiked() + 1));
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public PostEntity removeLiked(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setLiked((existsEntity.getLiked() - 1));
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public PostEntity addView(Long id) {
        final PostEntity existsEntity = get(id);
        existsEntity.setViewed((existsEntity.getViewed() + 1));
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public PostEntity update(Long id, PostEntity entity) {
        final PostEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public PostEntity delete(Long id) {
        final PostEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
