package com.example.demo.users.service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserTop;
import com.example.demo.users.repository.UserRepository;

import org.springframework.transaction.annotation.Transactional;

@Service
public class UserService {
    private final UserRepository repository;

    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    private void checkEmail(String email) {
        if (repository.findByEmailIgnoreCase(email).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("User with email %s is already exists", email));
        }
    }

    @Transactional(readOnly = true)
    public List<UserEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public UserEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(UserEntity.class, id));
    }

    @Transactional
    public UserEntity create(UserEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        checkEmail(entity.getEmail());
        repository.save(entity);
        return repository.save(entity);
    }

    @Transactional
    public UserEntity addFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        if (existsEntity.getFriends() == null)
            existsEntity.setFriends(new ArrayList<UserEntity>());
        existsEntity.getFriends().add(entity);
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public UserEntity removeFriend(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.getFriends().remove(entity);
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        checkEmail(entity.getEmail());
        existsEntity.setNickname(entity.getNickname());
        existsEntity.setEmail(entity.getEmail());
        existsEntity.setPassword(entity.getPassword());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public UserEntity delete(Long id) {
        final UserEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }

    @Transactional(readOnly = true)
    public Page<UserTop> getTop(int page, int size) {
        return repository.getFiveWithTopFriends(PageRequest.of(page, size));
    }
}
