package com.example.demo;

import java.util.Date;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class MessageServiceTests {
    @Autowired
    private UserService userService;
    @Autowired
    private MessageService messageService;

    private MessageEntity message;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> messageService.get(0L));
    }

    @BeforeEach
    void createData() {
        removeData();

        var user1 = userService.create(new UserEntity("sardq", "as2@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity("eafqr", "aasd@asd", "1234r", null));
        var user3 = userService.create(new UserEntity("asf", "aaasfsfs@asd", "332a", null));
        messageService.create(new MessageEntity(user1, user2, new Date(), "test"));
        messageService.create(new MessageEntity(user3, user2, new Date(), "asfgczv"));
        message = messageService
                .create(new MessageEntity(user2, user1, new Date(), "asvbsfd"));
    }

    @AfterEach
    void removeData() {
        messageService.getAll(0L).forEach(item -> messageService.delete(item.getId()));
        userService.getAll().forEach(item -> userService.delete(item.getId()));

    }

    @Test
    void createNullableTest() {
        final MessageEntity nullableType = new MessageEntity(null, null, null, null);
        Assertions.assertThrows(DataIntegrityViolationException.class, () -> messageService.create(nullableType));
    }

    @Test
    @Order(1)
    void createTest() {
        Assertions.assertEquals(3, messageService.getAll(0L).size());
    }

    @Test
    @Order(2)
    void updateTest() {
        final String test = "TEST";
        final MessageEntity entity = messageService.get(message.getId());
        final String oldText = entity.getText();
        final MessageEntity newEntity = messageService.update(message.getId(),
                new MessageEntity(
                        new UserEntity("safsaf", "afaa@asd", "qazq", null),
                        new UserEntity("safnkkjaslf", "afaafasd@asd", "2qhf", null),
                        new Date(), test));
        Assertions.assertEquals(3, messageService.getAll(0L).size());
        Assertions.assertEquals(test, newEntity.getText());
        Assertions.assertNotEquals(oldText, newEntity.getText());
    }

    @Test
    @Order(3)
    void deleteTest() {
        messageService.delete(message.getId());
        Assertions.assertEquals(2, messageService.getAll(0L).size());

        var user1 = userService.create(new UserEntity("sardq", "asafsd@asd", "asdsd", null));
        var user2 = userService.create(new UserEntity("sasfardq", "asfjsafcg@asd", "asfsaf", null));
        messageService.create(
                new MessageEntity(user1, user2, new Date(), "test"));
        Assertions.assertEquals(3, messageService.getAll(0L).size());
    }
}
