package com.example.demo;

import java.sql.Time;
import java.util.ArrayList;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.util.List;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.posts.model.PostEntity;
import com.example.demo.posts.service.PostService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserRole;
import com.example.demo.users.service.UserService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
    private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

    private final UserService userService;
    private final PostService postService;
    private final CommentService commentService;
    private final MessageService messageService;

    public DemoApplication(UserService userService, PostService postService, CommentService commentService,
            MessageService messageService) {
        this.userService = userService;
        this.postService = postService;
        this.commentService = commentService;
        this.messageService = messageService;
    }

    public static void main(String[] args) {
        SpringApplication.run(DemoApplication.class, args);
    }

    @Override
    public void run(String... args) throws Exception {
        if (args.length > 0 && Objects.equals("--populate", args[0])) {
            final var admin = new UserEntity("admin", "admin", "admin", null);
            admin.setRole(UserRole.ADMIN);
            userService.create(admin);

            log.info("Create default users values");
            final List<UserEntity> blank = new ArrayList<UserEntity>();
            final var user1 = userService.create(new UserEntity("sardq", "aasd@asd", "asdsd", null));
            userService.create(new UserEntity("Remy4508", "jbsdk@asd", "asdsd", null));
            userService.create(new UserEntity("Rand", "jbsdk@asd", "asdsd", null));
            final var user2 = userService.create(new UserEntity("Shirotame", "aasdsad@asd", "123", blank));
            final List<UserEntity> friends = new ArrayList<>();
            friends.add(user1);
            friends.add(user2);
            userService.addFriend(user1.getId(), user2);

            final var user3 = userService.create(new UserEntity("Domeztos", "aaxsd@asd", "asfgd", friends));
            final var user4 = userService.create(new UserEntity("Russel", "aacbxcsd@asd", "zxvb", blank));
            userService.addFriend(admin.getId(), user1);
            userService.addFriend(admin.getId(), user2);
            log.info("Create default posts values");
            final var post1 = postService.create(
                    new PostEntity(admin, new Time(System.currentTimeMillis()), "test", Long.valueOf(0),
                            Long.valueOf(0)));
            postService.create(
                    new PostEntity(user3, new Time(System.currentTimeMillis()), "test", Long.valueOf(11),
                            Long.valueOf(2)));
            postService.create(
                    new PostEntity(user4, new Time(System.currentTimeMillis()), "test", Long.valueOf(40),
                            Long.valueOf(15)));

            log.info("Create default comments values");
            commentService
                    .create(new CommentEntity(user1, post1, new Time(System.currentTimeMillis()), "my comment"));
            commentService
                    .create(new CommentEntity(user3, post1, new Time(System.currentTimeMillis()), "my comment2"));
            commentService
                    .create(new CommentEntity(user4, post1, new Time(System.currentTimeMillis()), "my comment3"));
            commentService
                    .create(new CommentEntity(user4, post1, new Time(System.currentTimeMillis()), "my comment3"));
            commentService
                    .create(new CommentEntity(user4, post1, new Time(System.currentTimeMillis()), "my comment3"));
            commentService
                    .create(new CommentEntity(user4, post1, new Time(System.currentTimeMillis()), "my comment3"));

            log.info("Create default messages values");
            messageService.create(new MessageEntity(admin, user2, new Time(System.currentTimeMillis()), "hello"));
            messageService
                    .create(new MessageEntity(admin, user2, new Time(System.currentTimeMillis()), "how are you"));
            messageService
                    .create(new MessageEntity(user2, admin, new Time(System.currentTimeMillis()), "I'm ok"));
        }
    }
}
