package com.example.demo.comments.service;

import java.util.List;
import java.util.stream.StreamSupport;
import java.sql.Time;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.repository.CommentRepository;
import com.example.demo.core.error.NotFoundException;

@Service
public class CommentService {
    private final CommentRepository repository;

    public CommentService(CommentRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<CommentEntity> getAll(int postId) {
        if (postId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else {
            return repository.findByPostId(postId);
        }

    }

    @Transactional(readOnly = true)
    public Page<CommentEntity> getAll(Long postId, int page, int size) {
        final Pageable pageRequest = PageRequest.of(page, size);
        if (postId <= 0L) {
            return repository.findAll(pageRequest);
        } else {
            return repository.findAllByPostId(postId, pageRequest);
        }
    }

    @Transactional(readOnly = true)
    public CommentEntity getOneByPostId(Long postId) {
        var list = repository.findByPostId(postId);
        return list.get(0);
    }

    @Transactional(readOnly = true)
    public Page<CommentEntity> getAllByParams(int page, int size, Long postId) {
        Pageable pageRequest = PageRequest.of(page, size);
        if (postId != null) {
            pageRequest = PageRequest.of(page, size);
            return repository.findAllByPostId(postId, pageRequest);
        }
        return repository.findAllComments(pageRequest);
    }

    @Transactional(readOnly = true)
    public CommentEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(CommentEntity.class, id));
    }

    @Transactional
    public CommentEntity create(CommentEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        entity.setTime(new Time(System.currentTimeMillis()));
        return repository.save(entity);
    }

    @Transactional
    public CommentEntity update(Long id, CommentEntity entity) {
        final CommentEntity existsEntity = get(id);
        existsEntity.setText(entity.getText());
        repository.save(existsEntity);
        return existsEntity;
    }

    @Transactional
    public CommentEntity delete(Long id) {
        final CommentEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
