package com.example.demo.messages.api;

import java.util.Map;
import java.sql.Time;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.convert;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;
import com.example.demo.users.api.UserDto;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(DialogController.URL)
public class DialogController {

    public static final String URL = "/dialogs";
    private static final String DIALOGS_VIEW = "dialogs";
    private static final String MESSAGE_VIEW = "dialog";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String MESSAGE_ATTRIBUTE = "message";

    private final MessageService messageService;
    private final UserService userService;
    private final ModelMapper modelMapper;

    public DialogController(MessageService messageService, ModelMapper modelMapper, UserService userService) {
        this.messageService = messageService;
        this.modelMapper = modelMapper;
        this.userService = userService;
    }

    private MessageDto toDto(MessageEntity entity) {
        var model = modelMapper.map(entity, MessageDto.class);
        convert time = new convert();
        model.setRecieverUsername(entity.getFriend().getUsername());
        model.setSenderUsername(entity.getPerson().getUsername());
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    private UserDto toUserDto(UserEntity entity) {
        var model = modelMapper.map(entity, UserDto.class);
        if (entity.getFriends() != null) {
            for (UserEntity user : entity.getFriends()) {
                model.addFriendId(user.getId());
            }
        }
        return model;

    }

    private MessageEntity toEntity(DialogDto dto) {
        final MessageEntity entity = modelMapper.map(dto, MessageEntity.class);
        entity.setFriend(userService.get(dto.getFriendId()));
        entity.setPerson(userService.get(dto.getPersonId()));
        return entity;

    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model,
            @AuthenticationPrincipal UserPrincipal userEntity) {
        var user = userService.get(userEntity.getId());
        model.addAttribute("principal", toUserDto(user));
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                userService.getByFriends(user.getFriends(), page, Constants.DEFUALT_PAGE_SIZE), this::toUserDto);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return DIALOGS_VIEW;
    }

    @GetMapping("/{id}")
    public String getDialogMessages(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        var user = userService.get(principal.getId());
        var friend = userService.get(id);
        Map<String, Object> attributes = PageAttributesMapper
                .toAttributes(messageService.getAll(principal.getId(), id, page, 5),
                        this::toDto);
        model.addAllAttributes(attributes);
        model.addAttribute("principal", toUserDto(user));
        model.addAttribute("user", toUserDto(friend));
        var newMessage = new MessageDto();
        newMessage.setPersonId(user.getId());
        newMessage.setFriendId(id);
        newMessage.setText("hello");
        model.addAttribute("chatmsg", newMessage);
        return MESSAGE_VIEW;
    }

    @PostMapping("{id}/edit")
    public String createDialogMessage(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = MESSAGE_ATTRIBUTE) @Valid DialogDto message,
            BindingResult bindingResult,
            @AuthenticationPrincipal UserPrincipal principal,
            Model model,
            RedirectAttributes redirectAttributes) {
        message.setPersonId(principal.getId());
        message.setSenderUsername(principal.getUsername());
        message.setFriendId(id);
        if (message.getText().length() < 4) {
            bindingResult.rejectValue("text", "profile:texts", "Неправильный текст.");
            model.addAttribute(MESSAGE_VIEW, message);
            return MESSAGE_VIEW;
        }
        var friend = userService.get(id);
        message.setRecieverUsername(friend.getUsername());
        convert conv = new convert();
        message.setTime(conv.dateToFormatDTOString(new Time(System.currentTimeMillis())));
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        messageService.create(toEntity(message));
        return Constants.REDIRECT_VIEW + "/dialogs/" + id;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        messageService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
