package com.example.demo.messages.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.convert;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.core.security.UserPrincipal;
import com.example.demo.messages.model.MessageEntity;
import com.example.demo.messages.service.MessageService;

@Controller
@RequestMapping(MessageController.URL)
public class MessageController {

    public static final String URL = Constants.ADMIN_PREFIX + "/message";
    private static final String MESSAGE_VIEW = "message";
    private static final String PAGE_ATTRIBUTE = "page";

    private final MessageService messageService;
    private final ModelMapper modelMapper;

    public MessageController(MessageService messageService, ModelMapper modelMapper) {
        this.messageService = messageService;
        this.modelMapper = modelMapper;
    }

    private MessageDto toDto(MessageEntity entity) {
        var model = modelMapper.map(entity, MessageDto.class);
        convert time = new convert();
        model.setRecieverUsername(entity.getFriend().getUsername());
        model.setSenderUsername(entity.getPerson().getUsername());
        model.setTime(time.dateToString(entity.getTime()));
        return model;
    }

    // private MessageEntity toEntity(MessageDto dto) {
    // final MessageEntity entity = modelMapper.map(dto, MessageEntity.class);
    // entity.setFriend(userService.get(dto.getFriendId()));
    // entity.setPerson(userService.get(dto.getPersonId()));
    // return entity;

    // }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model,
            @AuthenticationPrincipal UserPrincipal userEntity) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                messageService.getAll(0L, 0L, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return MESSAGE_VIEW;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        messageService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
