package com.example.demo.users.model;

import java.util.ArrayList;
import java.util.Objects;

import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;

import java.util.List;
import com.example.demo.core.model.BaseEntity;

import jakarta.persistence.CascadeType;
import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.FetchType;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.Table;

@Entity
@Table(name = "users")
public class UserEntity extends BaseEntity {
    @Column(nullable = false, length = 20)
    private String username;
    @Column(nullable = false, length = 20)
    private String email;
    @Column(nullable = false)
    private String password;
    private UserRole role;
    private Long isFriend;
    @ManyToMany(fetch = FetchType.EAGER, cascade = CascadeType.REMOVE)
    @OnDelete(action = OnDeleteAction.CASCADE)
    @JoinTable(name = "user_friendship", joinColumns = @JoinColumn(name = "first_user_id", referencedColumnName = "Id", nullable = false), inverseJoinColumns = @JoinColumn(name = "second_user_id", referencedColumnName = "Id", nullable = false))
    private List<UserEntity> friends = new ArrayList<UserEntity>();

    public UserEntity() {
    }

    public UserEntity(String username, String email, String password, List<UserEntity> friends) {
        this.username = username;
        this.email = email;
        this.password = password;
        this.friends = friends;
        this.role = UserRole.USER;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public UserRole getRole() {
        return role;
    }

    public void setRole(UserRole role) {
        this.role = role;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public List<UserEntity> getFriends() {
        return friends;
    }

    public void setIsFriend(Long isFriend) {
        this.isFriend = isFriend;
    }

    public Long getIsFriend() {
        return isFriend;
    }

    public void setFriends(List<UserEntity> friends) {
        this.friends = friends;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, username, email, password, friends);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final UserEntity other = (UserEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getFriends(), friends)
                && Objects.equals(other.getUsername(), username)
                && Objects.equals(other.getEmail(), email)
                && Objects.equals(other.getPassword(), password);
    }
}