package com.example.demo.users.repository;

import java.util.Optional;
import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.data.repository.PagingAndSortingRepository;
import org.springframework.data.repository.query.Param;

import com.example.demo.users.model.FriendsEntity;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.model.UserTop;

public interface UserRepository extends CrudRepository<UserEntity, Long>,
        PagingAndSortingRepository<UserEntity, Long> {
    List<UserEntity> findByUsername(String username);

    Page<UserEntity> findByUsername(String username, Pageable pageable);

    Optional<UserEntity> findById(Long Id);

    @Query("SELECT distinct f FROM UserEntity u JOIN u.friends f  WHERE f.id IN :friends")
    Page<UserEntity> findByFriends(@Param("friends") List<Long> friends, Pageable pageable);

    @Query("SELECT u as user, CASE WHEN :uId = f.id THEN 1 ELSE 0 END as isFriend FROM UserEntity u LEFT JOIN u.friends f Where :uId != u.id ")
    Page<FriendsEntity> findFriendsById(@Param("uId") Long id, Pageable pageable);

    Optional<UserEntity> findByUsernameIgnoreCase(String username);

    @Query("select "
            + " u.username as user, coalesce(size(u.friends), 0) as count  "
            + "from UserEntity u "
            + "order by size(u.friends) desc")
    Page<UserTop> getFiveWithTopFriends(Pageable pageable);
}
